==> neuralnoise-0.6.0-py3-none-any.whl: neuralnoise/studio/create.py <==
import hashlib
import json
import logging
from pathlib import Path
from typing import Any, Literal

from pydub import AudioSegment
from pydub.effects import normalize
from rich.progress import Progress

from neuralnoise.studio import PodcastStudio
from neuralnoise.tts import generate_audio_segment
from neuralnoise.types import StudioConfig

logging.basicConfig(
    level=logging.INFO, format="%(asctime)s - %(levelname)s - %(message)s"
)
logger = logging.getLogger(__name__)


def create_podcast_episode_from_script(
    script: dict[str, Any], config: StudioConfig, output_dir: Path
) -> AudioSegment:
    script_segments = []

    temp_dir = output_dir / "segments"
    temp_dir.mkdir(exist_ok=True)

    sections_ids = list(sorted(script["sections"].keys()))
    script_segments = [
        (section_id, segment)
        for section_id in sections_ids
        for segment in script["sections"][section_id]["segments"]
    ]

    with Progress() as progress:
        task = progress.add_task(
            "[cyan]Generating audio segments...", total=len(script_segments)
        )
        audio_segments = []

        for section_id, segment in script_segments:
            speaker = config.speakers[segment["speaker"]]
            content = segment["content"]

            content = content.replace("¡", "").replace("¿", "")

            content_hash = hashlib.md5(content.encode("utf-8")).hexdigest()
            segment_path = temp_dir / f"{section_id}_{segment['id']}_{content_hash}.mp3"

            audio_segment = generate_audio_segment(
                content, speaker, output_path=segment_path
            )

            audio_segments.append(audio_segment)

            if blank_duration := segment.get("blank_duration"):
                silence = AudioSegment.silent(duration=blank_duration * 1000)
                audio_segments.append(silence)

            progress.update(task, advance=1)

    podcast = AudioSegment.empty()

    for chunk in audio_segments:
        podcast += chunk

    podcast = normalize(podcast)

    return podcast


def create_podcast_episode(
    name: str,
    content: str,
    config: StudioConfig | None = None,
    config_path: str | Path | None = None,
    format: Literal["wav", "mp3", "ogg"] = "wav",
    only_script: bool = False,
):
    # Create output directory
    output_dir = Path("output") / name
    output_dir.mkdir(parents=True, exist_ok=True)

    # Load configuration
    if config_path:
        logger.info("🔧  Loading configuration from %s", config_path)
        with open(config_path, "r") as f:
            config = StudioConfig.model_validate_json(f.read())

    if not config:
        raise ValueError("No studio configuration provided")

    # Generate the script
    script_path = output_dir / "script.json"

    if script_path.exists():
        logger.info("💬  Loading cached script")
        script = json.loads(script_path.read_text(encoding="utf-8"))
    else:
        logger.info("💬  Generating podcast script")
        studio = PodcastStudio(work_dir=output_dir, config=config)
        script = studio.generate_script(content)

        script_path.write_text(json.dumps(script), encoding="utf-8")

    if only_script:
        return

    # Generate audio segments and create the podcast
    logger.info("🎙️  Recording podcast episode")
    podcast = create_podcast_episode_from_script(script, config, output_dir=output_dir)

    # Export podcast
    podcast_filepath = output_dir / f"output.{format}"
    logger.info("️💾  Exporting podcast to %s", podcast_filepath)
    podcast.export(podcast_filepath, format=format)

    logger.info("✅  Podcast generation complete")

==> neuralnoise-0.9.0-py3-none-any.whl: neuralnoise/studio/create.py <==
import hashlib
import json
import logging
from pathlib import Path
from typing import Any, Literal

from pydub import AudioSegment
from pydub.effects import normalize
from rich.progress import track

from neuralnoise.studio import PodcastStudio
from neuralnoise.tts import generate_audio_segment
from neuralnoise.types import StudioConfig


logger = logging.getLogger(__name__)


def create_podcast_episode_from_script(
    script: dict[str, Any], config: StudioConfig, output_dir: Path
) -> AudioSegment:
    script_segments = []

    temp_dir = output_dir / "segments"
    temp_dir.mkdir(exist_ok=True)

    sections_ids = list(sorted(script["sections"].keys()))
    script_segments = [
        (section_id, segment)
        for section_id in sections_ids
        for segment in script["sections"][section_id]["segments"]
    ]

    audio_segments = []

    for section_id, segment in track(
        script_segments,
        description="Generating audio segments...",
        total=len(script_segments),
    ):
        speaker = config.speakers[segment["speaker"]]
        content = segment["content"]

        content = content.replace("¡", "").replace("¿", "")

        content_hash = hashlib.md5(content.encode("utf-8")).hexdigest()
        segment_path = temp_dir / f"{section_id}_{segment['id']}_{content_hash}.mp3"

        audio_segment = generate_audio_segment(
            content, speaker, output_path=segment_path
        )

        audio_segments.append(audio_segment)

        if blank_duration := segment.get("blank_duration"):
            silence = AudioSegment.silent(duration=blank_duration * 1000)
            audio_segments.append(silence)

    podcast = AudioSegment.empty()

    for chunk in audio_segments:
        podcast += chunk

    podcast = normalize(podcast)

    return podcast


def create_podcast_episode(
    name: str,
    content: str,
    config: StudioConfig | None = None,
    config_path: str | Path | None = None,
    format: Literal["wav", "mp3", "ogg"] = "wav",
    only_script: bool = False,
):
    # Create output directory
    output_dir = Path("output") / name
    output_dir.mkdir(parents=True, exist_ok=True)

    # Load configuration
    if config_path:
        logger.info("🔧  Loading configuration from %s", config_path)
        with open(config_path, "r") as f:
            config = StudioConfig.model_validate_json(f.read())

    if not config:
        raise ValueError("No studio configuration provided")

    # Generate the script
    script_path = output_dir / "script.json"

    if script_path.exists():
        logger.info("💬  Loading cached script")
        script = json.loads(script_path.read_text())
    else:
        logger.info("💬  Generating podcast script")
        studio = PodcastStudio(work_dir=output_dir, config=config)
        script = studio.generate_script(content)

        script_path.write_text(json.dumps(script, ensure_ascii=False))

    if only_script:
        return

    # Generate audio segments and create the podcast
    logger.info("🎙️  Recording podcast episode")
    podcast = create_podcast_episode_from_script(script, config, output_dir=output_dir)

    # Export podcast
    podcast_filepath = output_dir / f"output.{format}"
    logger.info("️💾  Exporting podcast to %s", podcast_filepath)
    podcast.export(podcast_filepath, format=format)

    logger.info("✅  Podcast generation complete")
